.x-tree-checkbox {
	background:url('/static/ext/resources/images/default/form/checkbox.gif') no-repeat 0 0;
	height:13px;
	width:13px;
	margin:0 1px;
	vertical-align:middle;
}
.x-tree-checkbox-over .x-tree-checkbox {
	background-position:-13px 0;
}
.x-tree-checkbox-down .x-tree-checkbox {
	background-position:-26px 0;
}
.x-tree-node-disabled .x-tree-checkbox {
	background-position:-39px 0;
}
.x-tree-node-checked {
	background-position:0 -13px;
}
.x-tree-checkbox-over .x-tree-node-checked {
	background-position:-13px -13px;
}
.x-tree-checkbox-down .x-tree-node-checked {
	background-position:-26px -13px;
}
.x-tree-node-disabled .x-tree-node-checked {
	background-position:-39px -13px;
}
.x-tree-node-grayed {
	background-position:0 -26px;
}
.x-tree-checkbox-over .x-tree-node-grayed {
	background-position:-13px -26px;
}
.x-tree-checkbox-down .x-tree-node-grayed {
	background-position:-26px -26px;
}
.x-tree-node-disabled .x-tree-node-grayed {
	background-position:-39px -26px;
}

/* eof */
